@@ -19,7 +19,7 @@ class LocalFileSystem(AbstractFileSystem):
 
     Parameters
     ----------
-    auto_mkdirs: bool
+    auto_mkdir: bool
         Whether, when opening a file, the directory containing it should
         be created (if it doesn't already exist). This is assumed by pyarrow
         code.
@@ -184,7 +184,7 @@ class LocalFileSystem(AbstractFileSystem):
         path = stringify_path(path)
         if path.startswith("file://"):
             path = path[7:]
-        return make_path_posix(path).rstrip("/")
+        return make_path_posix(path).rstrip("/") or cls.root_marker
 
     def _isfilestore(self):
         # Inheriting from DaskFileSystem makes this False (S3, etc. were)
