@@ -2,8 +2,11 @@ import base64
 import io
 import itertools
 import logging
+import os
 from functools import lru_cache
 
+import fsspec.core
+
 try:
     import ujson as json
 except ImportError:
@@ -36,7 +39,7 @@ class ReferenceFileSystem(AsyncFileSystem):
     Reference dict format:
     {path0: bytes_data, path1: (target_url, offset, size)}
 
-    https://github.com/intake/fsspec-reference-maker/blob/main/README.md
+    https://github.com/fsspec/kerchunk/blob/main/README.md
     """
 
     protocol = "reference"
@@ -52,7 +55,7 @@ class ReferenceFileSystem(AsyncFileSystem):
         remote_options=None,
         fs=None,
         template_overrides=None,
-        simple_templates=False,
+        simple_templates=True,
         loop=None,
         ref_type=None,
         **kwargs,
@@ -77,7 +80,9 @@ class ReferenceFileSystem(AsyncFileSystem):
             Extra FS options for loading the reference file, if given as a path
         remote_protocol : str
             The protocol of the filesystem on which the references will be evaluated
-            (unless fs is provided)
+            (unless fs is provided). If not given, will be derived from the first
+            URL that has a protocol in the templates or in the references, in that
+            order.
         remote_options : dict
             kwargs to go with remote_protocol
         fs : file system instance
@@ -90,6 +95,9 @@ class ReferenceFileSystem(AsyncFileSystem):
             If None, guessed from URL suffix, defaulting to JSON. Ignored if fo
             is not a string.
         simple_templates: bool
+            Whether templates can be processed with simple replace (True) or if
+            jinja  is needed (False, much slower). All reference sets produced by
+            ``kerchunk`` are simple in this sense, but the spec allows for complex.
         kwargs : passed to parent class
         """
         super().__init__(loop=loop, **kwargs)
@@ -97,6 +105,7 @@ class ReferenceFileSystem(AsyncFileSystem):
         self.dataframe = False
         self.template_overrides = template_overrides
         self.simple_templates = simple_templates
+        self.templates = {}
         if hasattr(fo, "read"):
             text = fo.read()
         elif isinstance(fo, str):
@@ -140,11 +149,30 @@ class ReferenceFileSystem(AsyncFileSystem):
             self._process_dataframe()
         else:
             self._process_references(text, template_overrides)
-        if fs is None and remote_protocol is None:
+        if fs is not None:
+            self.fs = fs
+            return
+        if remote_protocol is None:
+            for ref in self.templates.values():
+                if callable(ref):
+                    ref = ref()
+                protocol, _ = fsspec.core.split_protocol(ref)
+                if protocol:
+                    remote_protocol = protocol
+                    break
+        if remote_protocol is None:
+            for ref in self.references.values():
+                if callable(ref):
+                    ref = ref()
+                if isinstance(ref, list) and ref[0]:
+                    protocol, _ = fsspec.core.split_protocol(ref[0])
+                    if protocol:
+                        remote_protocol = protocol
+                        break
+        if remote_protocol is None:
             remote_protocol = target_protocol
-        if remote_protocol:
-            fs = filesystem(remote_protocol, loop=loop, **(remote_options or {}))
-        self.fs = fs
+
+        self.fs = filesystem(remote_protocol, loop=loop, **(remote_options or {}))
 
     @property
     def loop(self):
@@ -193,6 +221,7 @@ class ReferenceFileSystem(AsyncFileSystem):
         part_or_url, start0, end0 = self._cat_common(path)
         if isinstance(part_or_url, bytes):
             return part_or_url[start:end]
+        # TODO: update start0, end0 if start/end given, instead of slicing
         return self.fs.cat_file(part_or_url, start=start0, end=end0)[start:end]
 
     def pipe_file(self, path, value, **_):
@@ -200,21 +229,25 @@ class ReferenceFileSystem(AsyncFileSystem):
         self.references[path] = value
 
     async def _get_file(self, rpath, lpath, **kwargs):
+        if self.isdir(rpath):
+            return os.makedirs(lpath, exist_ok=True)
         data = await self._cat_file(rpath)
         with open(lpath, "wb") as f:
             f.write(data)
 
     def get_file(self, rpath, lpath, callback=_DEFAULT_CALLBACK, **kwargs):
+        if self.isdir(rpath):
+            return os.makedirs(lpath, exist_ok=True)
         data = self.cat_file(rpath, **kwargs)
-        callback.lazy_call("set_size", len, data)
+        callback.set_size(len(data))
         with open(lpath, "wb") as f:
             f.write(data)
-        callback.lazy_call("absolute_update", len, data)
+        callback.absolute_update(len(data))
 
     def get(self, rpath, lpath, recursive=False, **kwargs):
         if self.fs.async_impl:
             return sync(self.loop, self._get, rpath, lpath, recursive, **kwargs)
-        return AbstractFileSystem.get(rpath, lpath, recursive=recursive, **kwargs)
+        return AbstractFileSystem.get(self, rpath, lpath, recursive=recursive, **kwargs)
 
     def cat(self, path, recursive=False, **kwargs):
         if self.fs.async_impl:
@@ -247,7 +280,6 @@ class ReferenceFileSystem(AsyncFileSystem):
 
         if self.templates:
             self.df["url"] = self.df["url"].map(_render_jinja)
-        self._dircache_from_items()
 
     def _process_references(self, references, template_overrides=None):
         if isinstance(references, (str, bytes)):
@@ -261,7 +293,6 @@ class ReferenceFileSystem(AsyncFileSystem):
             raise ValueError(f"Unknown reference spec version: {vers}")
         # TODO: we make dircache by iterating over all entries, but for Spec >= 1,
         #  can replace with programmatic. Is it even needed for mapper interface?
-        self._dircache_from_items()
 
     def _process_references0(self, references):
         """Make reference dict for Spec Version 0"""
@@ -271,10 +302,11 @@ class ReferenceFileSystem(AsyncFileSystem):
         self.references = references
 
     def _process_references1(self, references, template_overrides=None):
-        try:
-            import jinja2
-        except ImportError as e:
-            raise ValueError("Reference Spec Version 1 requires jinja2") from e
+        if not self.simple_templates or self.templates:
+            try:
+                import jinja2
+            except ImportError as e:
+                raise ValueError("Reference Spec Version 1 requires jinja2") from e
         self.references = {}
         self._process_templates(references.get("templates", {}))
 
@@ -287,7 +319,7 @@ class ReferenceFileSystem(AsyncFileSystem):
                 if v.startswith("base64:"):
                     self.references[k] = base64.b64decode(v[7:])
                 self.references[k] = v
-            else:
+            elif self.templates:
                 u = v[0]
                 if "{{" in u:
                     if self.simple_templates:
@@ -299,6 +331,8 @@ class ReferenceFileSystem(AsyncFileSystem):
                     else:
                         u = _render_jinja(u)
                 self.references[k] = [u] if len(v) == 1 else [u, v[1], v[2]]
+            else:
+                self.references[k] = v
         self.references.update(self._process_gen(references.get("gen", [])))
 
     def _process_templates(self, tmp):
@@ -369,14 +403,14 @@ class ReferenceFileSystem(AsyncFileSystem):
                     size = None
                 else:
                     _, start, size = part
-            par = self._parent(path)
+            par = path.rsplit("/", 1)[0] if "/" in path else ""
             par0 = par
             while par0 and par0 not in self.dircache:
                 # build parent directories
                 self.dircache[par0] = []
-                self.dircache.setdefault(self._parent(par0), []).append(
-                    {"name": par0, "type": "directory", "size": 0}
-                )
+                self.dircache.setdefault(
+                    par0.rsplit("/", 1)[0] if "/" in par0 else "", []
+                ).append({"name": par0, "type": "directory", "size": 0})
                 par0 = self._parent(par0)
 
             self.dircache[par].append({"name": path, "type": "file", "size": size})
@@ -389,6 +423,8 @@ class ReferenceFileSystem(AsyncFileSystem):
 
     def ls(self, path, detail=True, **kwargs):
         path = self._strip_protocol(path)
+        if not self.dircache:
+            self._dircache_from_items()
         out = self._ls_from_cache(path)
         if out is None:
             raise FileNotFoundError
@@ -397,20 +433,30 @@ class ReferenceFileSystem(AsyncFileSystem):
         return [o["name"] for o in out]
 
     def exists(self, path, **kwargs):  # overwrite auto-sync version
-        try:
-            return self._ls_from_cache(path) is not None
-        except FileNotFoundError:
-            return False
+        return self.isdir(path) or self.isfile(path)
 
     def isdir(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "directory"
+        if self.dircache:
+            return path in self.dircache
+        else:
+            # this may be faster than building dircache for single calls, but
+            # by looping will be slow for many calls; could cache it?
+            return any(_.startswith(f"{path}/") for _ in self.references)
 
     def isfile(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "file"
+        return path in self.references
 
     async def _ls(self, path, detail=True, **kwargs):  # calls fast sync code
         return self.ls(path, detail, **kwargs)
 
+    def find(self, path, maxdepth=None, withdirs=False, **kwargs):
+        if withdirs:
+            return super().find(path, maxdepth=maxdepth, withdirs=withdirs, **kwargs)
+        if path:
+            path = self._strip_protocol(path)
+            return sorted(k for k in self.references if k.startswith(path))
+        return sorted(self.references)
+
     def info(self, path, **kwargs):
         out = self.ls(path, True)
         out0 = [o for o in out if o["name"] == path]
